@@ -1,3 +1,5 @@
+//! # Decoding BER
+
 mod config;
 mod error;
 pub(super) mod parser;
@@ -8,7 +10,12 @@ use snafu::*;
 
 use super::identifier::Identifier;
 use crate::{
-    types::{self, Tag},
+    de::Error as _,
+    types::{
+        self,
+        oid::{MAX_OID_FIRST_OCTET, MAX_OID_SECOND_OCTET},
+        Tag,
+    },
     Decode,
 };
 
@@ -18,24 +25,19 @@ type Result<T, E = Error> = core::result::Result<T, E>;
 
 const EOC: &[u8] = &[0, 0];
 
+/// A BER and variants decoder. Capable of decoding BER, CER, and DER.
 pub struct Decoder<'input> {
     input: &'input [u8],
     config: DecoderOptions,
 }
 
 impl<'input> Decoder<'input> {
+    /// Create a new [`Decoder`] from the given `input` and `config`.
     pub fn new(input: &'input [u8], config: DecoderOptions) -> Self {
         Self { input, config }
     }
 
-    pub fn is_eoc(&mut self) -> bool {
-        self.input.is_empty()
-            || nom::bytes::streaming::tag::<_, _, (_, _)>(EOC)(self.input)
-                .ok()
-                .is_some()
-    }
-
-    pub fn parse_eoc(&mut self) -> Result<()> {
+    fn parse_eoc(&mut self) -> Result<()> {
         let (i, _) = nom::bytes::streaming::tag(EOC)(self.input).map_err(error::map_nom_err)?;
         self.input = i;
         Ok(())
@@ -43,14 +45,14 @@ impl<'input> Decoder<'input> {
 
     pub(crate) fn parse_value(&mut self, tag: Tag) -> Result<(Identifier, Option<&'input [u8]>)> {
         let (input, (identifier, contents)) =
-            self::parser::parse_value(&self.config, self.input, tag)?;
+            self::parser::parse_value(&self.config, self.input, Some(tag))?;
         self.input = input;
         Ok((identifier, contents))
     }
 
     pub(crate) fn parse_primitive_value(&mut self, tag: Tag) -> Result<(Identifier, &'input [u8])> {
         let (input, (identifier, contents)) =
-            self::parser::parse_value(&self.config, self.input, tag)?;
+            self::parser::parse_value(&self.config, self.input, Some(tag))?;
         self.input = input;
         match contents {
             Some(contents) => Ok((identifier, contents)),
@@ -58,27 +60,59 @@ impl<'input> Decoder<'input> {
         }
     }
 
-    pub(crate) fn peek_identifier(&self) -> Result<Identifier> {
-        let (_, identifier) =
-            self::parser::parse_identifier_octet(self.input).map_err(error::map_nom_err)?;
-        Ok(identifier)
+    /// Parses a constructed ASN.1 value, checking the `tag`, and optionally
+    /// checking if the identifier is marked as encoded. This should be true
+    /// in all cases except explicit prefixes.
+    fn parse_constructed_contents<D, F>(
+        &mut self,
+        tag: Tag,
+        check_identifier: bool,
+        decode_fn: F,
+    ) -> Result<D>
+    where
+        F: FnOnce(&mut Self) -> Result<D>,
+    {
+        let (identifier, contents) = self.parse_value(tag)?;
+
+        error::assert_tag(tag, identifier.tag)?;
+
+        if check_identifier && identifier.is_primitive() {
+            return Err(Error::custom("Invalid constructed identifier"));
+        }
+
+        let (streaming, contents) = match contents {
+            Some(contents) => (false, contents),
+            None => (true, self.input),
+        };
+
+        let mut inner = Self::new(contents, self.config);
+
+        let result = (decode_fn)(&mut inner)?;
+
+        if streaming {
+            self.input = inner.input;
+            self.parse_eoc()?;
+        } else if !inner.input.is_empty() {
+            return Err(Error::UnexpectedExtraData {
+                length: inner.input.len(),
+            });
+        }
+
+        Ok(result)
     }
 }
 
 impl<'input> crate::Decoder for Decoder<'input> {
     type Error = Error;
 
-    fn peek_tag(&self) -> Result<Tag> {
-        Ok(self.peek_identifier()?.tag)
-    }
-
-    fn decode_any(&mut self, tag: Tag) -> Result<Vec<u8>> {
-        let (input, (_, contents)) = self::parser::parse_value(&self.config, self.input, tag)?;
+    fn decode_any(&mut self) -> Result<types::Any> {
+        let (input, _) = self::parser::parse_value(&self.config, self.input, None)?;
+        let diff = self.input.len() - input.len();
+        let contents = &self.input[..diff];
         self.input = input;
 
-        Ok(match contents {
-            Some(c) => c.to_vec(),
-            None => input.to_vec(),
+        Ok(types::Any {
+            contents: contents.to_vec(),
         })
     }
 
@@ -161,17 +195,24 @@ impl<'input> crate::Decoder for Decoder<'input> {
         use num_traits::ToPrimitive;
         let contents = self.parse_primitive_value(tag)?.1;
         let (mut contents, root_octets) =
-            parser::parse_encoded_number(contents).map_err(error::map_nom_err)?;
-        let second = (&root_octets % 40u8)
-            .to_u32()
-            .context(error::IntegerOverflow { max_width: 32u32 })?;
-        let first = ((root_octets - second) / 40u8)
+            parser::parse_base128_number(contents).map_err(error::map_nom_err)?;
+        let the_number = root_octets
             .to_u32()
             .context(error::IntegerOverflow { max_width: 32u32 })?;
+        let first: u32;
+        let second: u32;
+        const MAX_OID_THRESHOLD: u32 = MAX_OID_SECOND_OCTET + 1;
+        if the_number > MAX_OID_FIRST_OCTET * MAX_OID_THRESHOLD + MAX_OID_SECOND_OCTET {
+            first = MAX_OID_FIRST_OCTET;
+            second = the_number - MAX_OID_FIRST_OCTET * MAX_OID_THRESHOLD;
+        } else {
+            second = the_number % MAX_OID_THRESHOLD;
+            first = (the_number - second) / MAX_OID_THRESHOLD;
+        }
         let mut buffer = alloc::vec![first, second];
 
         while !contents.is_empty() {
-            let (c, number) = parser::parse_encoded_number(contents).map_err(error::map_nom_err)?;
+            let (c, number) = parser::parse_base128_number(contents).map_err(error::map_nom_err)?;
             contents = c;
             buffer.push(
                 number
@@ -211,7 +252,7 @@ impl<'input> crate::Decoder for Decoder<'input> {
                             Ok(string)
                         }
                     }
-                    _ => return Err(Error::InvalidBitString { bits: unused_bits }),
+                    _ => Err(Error::InvalidBitString { bits: unused_bits }),
                 }
             })?;
 
@@ -238,17 +279,18 @@ impl<'input> crate::Decoder for Decoder<'input> {
 
     fn decode_generalized_time(&mut self, tag: Tag) -> Result<types::GeneralizedTime> {
         let string = self.decode_utf8_string(tag)?;
-        types::GeneralizedTime::parse_from_rfc3339(&string)
+        chrono::NaiveDateTime::parse_from_str(&string, "%Y%m%d%H%M%SZ")
             .ok()
             .context(error::InvalidDate)
+            .map(|date| types::GeneralizedTime::from_utc(date, chrono::FixedOffset::east(0)))
     }
 
     fn decode_utc_time(&mut self, tag: Tag) -> Result<types::UtcTime> {
         let string = self.decode_utf8_string(tag)?;
-        types::GeneralizedTime::parse_from_rfc2822(&string)
-            .map(types::UtcTime::from)
+        chrono::NaiveDateTime::parse_from_str(&string, "%y%m%d%H%M%SZ")
             .ok()
             .context(error::InvalidDate)
+            .map(|date| types::UtcTime::from_utc(date, chrono::Utc))
     }
 
     fn decode_sequence_of<D: Decode>(&mut self, tag: Tag) -> Result<Vec<D>, Self::Error> {
@@ -263,32 +305,45 @@ impl<'input> crate::Decoder for Decoder<'input> {
         })
     }
 
+    fn decode_set_of<D: Decode + Ord>(&mut self, tag: Tag) -> Result<types::SetOf<D>, Self::Error> {
+        self.decode_sequence(tag, |decoder| {
+            let mut items = types::SetOf::new();
+
+            while let Ok(item) = D::decode(decoder) {
+                items.insert(item);
+            }
+
+            Ok(items)
+        })
+    }
+
     fn decode_sequence<D, F: FnOnce(&mut Self) -> Result<D>>(
         &mut self,
         tag: Tag,
         decode_fn: F,
     ) -> Result<D> {
-        let contents = self.parse_value(tag)?.1;
-
-        let (streaming, contents) = match contents {
-            Some(contents) => (false, contents),
-            None => (true, self.input),
-        };
-
-        let mut inner = Self::new(contents, self.config);
-
-        let result = (decode_fn)(&mut inner)?;
-
-        if streaming {
-            self.input = inner.input;
-            self.parse_eoc()?;
-        }
-
-        Ok(result)
+        self.parse_constructed_contents(tag, true, decode_fn)
     }
 
     fn decode_explicit_prefix<D: Decode>(&mut self, tag: Tag) -> Result<D> {
-        self.decode_sequence(tag, D::decode)
+        self.parse_constructed_contents(tag, false, D::decode)
+    }
+
+    fn decode_set<FIELDS, SET, F>(&mut self, tag: Tag, decode_fn: F) -> Result<SET, Self::Error>
+    where
+        SET: Decode,
+        FIELDS: Decode,
+        F: FnOnce(Vec<FIELDS>) -> Result<SET, Self::Error>,
+    {
+        self.decode_sequence(tag, |decoder| {
+            let mut fields = Vec::new();
+
+            while let Ok(value) = FIELDS::decode(decoder) {
+                fields.push(value);
+            }
+
+            (decode_fn)(fields)
+        })
     }
 }
 
@@ -434,11 +489,11 @@ mod tests {
 
     #[test]
     fn sequence() {
-        use types::IA5String;
+        use types::Ia5String;
         // Taken from examples in 8.9 of X.690.
         #[derive(Debug, PartialEq)]
         struct Foo {
-            name: IA5String,
+            name: Ia5String,
             ok: bool,
         }
 
@@ -452,7 +507,7 @@ mod tests {
                 tag: Tag,
             ) -> Result<Self, D::Error> {
                 decoder.decode_sequence(tag, |sequence| {
-                    let name: IA5String = IA5String::decode(sequence)?;
+                    let name: Ia5String = Ia5String::decode(sequence)?;
                     let ok: bool = bool::decode(sequence)?;
                     Ok(Self { name, ok })
                 })
@@ -465,7 +520,7 @@ mod tests {
         };
         let bytes = &[
             0x30, 0x0A, // TAG + LENGTH
-            0x16, 0x05, 0x53, 0x6d, 0x69, 0x74, 0x68, // IA5String "Smith"
+            0x16, 0x05, 0x53, 0x6d, 0x69, 0x74, 0x68, // Ia5String "Smith"
             0x01, 0x01, 0xff, // BOOL True
         ];
 
@@ -516,4 +571,27 @@ mod tests {
             0x50, 0x10, 0x10, 0x10,
         ]);
     }
+
+    #[test]
+    fn any() {
+        let expected = &[0x1A, 0x05, 0x4A, 0x6F, 0x6E, 0x65, 0x73];
+        assert_eq!(
+            Any {
+                contents: expected.to_vec()
+            },
+            decode(expected).unwrap()
+        );
+    }
+
+    #[test]
+    fn decoding_oid() {
+        use crate::Decoder;
+
+        let mut decoder =
+            super::Decoder::new(&[0x06, 0x03, 0x88, 0x37, 0x01], DecoderOptions::der());
+        let oid = decoder.decode_object_identifier(Tag::OBJECT_IDENTIFIER);
+        assert!(oid.is_ok());
+        let oid = oid.unwrap();
+        assert_eq!(ObjectIdentifier::new([2, 999, 1].to_vec()).unwrap(), oid);
+    }
 }
