@@ -1,13 +1,18 @@
 use snafu::*;
 
+/// An error that ocurred when encoding BER or any of its variants.
 #[derive(Snafu)]
 #[snafu(visibility = "pub(crate)")]
 #[derive(Debug)]
 pub enum Error {
-    #[snafu(display("OBJECT IDENTIFIER must have at least two components."))]
+    /// `OBJECT IDENTIFIER` must have at least two components.
     InvalidObjectIdentifier,
+    /// A custom error.
     #[snafu(display("Custom Error:\n{}", msg))]
-    Custom { msg: alloc::string::String },
+    Custom {
+        /// The custom error's message.
+        msg: alloc::string::String,
+    },
 }
 
 impl crate::enc::Error for Error {
